//*****************************************************************************
//*
//*
//*		CDiskDoc  Klasse
//*
//*
//*****************************************************************************
#if !defined(AFX_DISKDOC_H__984867C9_D3DA_51D1_3271_243753540000__INCLUDED_)
#define      AFX_DISKDOC_H__984867C9_D3DA_51D1_3271_243753540000__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif

#define 	TYPE_MBR		1
#define 	TYPE_BOOT		2
#define 	TYPE_BOOT32		3
#define 	TYPE_BOOTNTFS	4
#define 	TYPE_BOOTEXFAT	5
#define 	TYPE_DIR		6
#define 	TYPE_INFO		7
#define 	TYPE_FAT12 		8
#define 	TYPE_FAT16		9
#define 	TYPE_FAT32		10
#define 	TYPE_FATEX		11
#define 	TYPE_VDESC		12
#define 	TYPE_PTABLE		13
#define 	TYPE_CDDIR		14
#define 	TYPE_MFT		15
#define 	TYPE_MFTMIR		16
#define 	TYPE_ECMA		17
#define 	TYPE_GPT		18
#define 	TYPE_GPTLIST	19
#define 	TYPE_EXFATDIR	20
#define 	TYPE_SUPERBLK	21

#define 	UNITS_MAX		128
#define 	PARTS_MAX		8

class	CProgressDlg;
class	CHexFind;

#include	"DiskIO.h"

//*****************************************************************************
//*
//*		Strukturen
//*
//*****************************************************************************

#pragma pack(push,1)

typedef struct
	{
	char    cFileName[8];
	char    cExtension[3];
	BYTE    bAttributes;
	BYTE    bNTReserved;
	BYTE    bCreateTimeTenthSecond;		// Range 0..199 + 1st Sec
	WORD	wCreateTime;				// 0-4=Sec*2 5-11=Min 11-15=Hour
	WORD	wCreateDate;				// 0-4=Day   5-8=Month 9-15=Year+1980
	WORD	wLastAccessDate;			// 0-4=Day   5-8=Month 9-15=Year+1980
	WORD    wFirstClusterHi;			// FAT-32 only
	WORD	wWriteTime;					// 0-4=Sec*2 5-11=Min 11-15=Hour
	WORD	wWriteDate;					// 0-4=Day   5-8=Month 9-15=Year+1980
	WORD    wFirstCluster;
	DWORD   dwFileSize;
	}DirEntry;

typedef struct
	{
	BYTE	bOrd;          				// sequency number (backwards, 0x40 set if last record)
	WORD	wName1[5];     				// name chars 1..5
	BYTE	bAttributes;   				// magic value
	BYTE	bType;         				// must be zero
	BYTE	bCheckSum;     				// checksum of the short name
	WORD	wName2[6];     				// name chars 6..11
	WORD	wFirstCluster; 				// zero
	WORD	wName3[2];					// name chars 12..13
	}LfnDirEntry;

typedef struct
	{
	BYTE	bBootIndicator;       		// 0x80 for bootable, 0 otherwise
    BYTE	bStartHead;           		// 0 based
    BYTE	bStartSector;         		// 1 based, bits 0..5, bits 6,7 are bits 8,9 of StartTrack
    BYTE	bStartTrack;          		// 0 based, bits 0..7
    BYTE	bOSType;              		// FAT-12: 1, FAT-16: 4, 6, 14, FAT-32: 11, 12
    BYTE	bEndHead;             		// see StartHead
    BYTE	bEndSector;           		// see StartSector
    BYTE	bEndTrack;					// see StartTrack
	DWORD	dwRelativeSector;			// offset to first sector of partition data
    DWORD	dwSectors;					// number of sectors in partition
	}PartitionRecord;

typedef struct
	{
	BYTE			bBootCode[440];
	DWORD			dwSerialNumber;
	WORD			wFree;
	PartitionRecord sTable[4];
	WORD			wSignature;			// 0xAA55
	}MasterBootRecord;

typedef struct
	{
	BYTE	aSignatur[8];				// "EFI PART"
	BYTE	aRevision[4];				// 00h 00h 01h 00h
	DWORD	dwHeaderSize;				// Header-Gre - Little Endian (5Ch 00h 00h 00h entspricht 92 bytes)
	DWORD	dwHeaderCrc;				// Header-CRC32-Prfsumme
	DWORD	dwReserved;					// Reservierter Bereich - muss Null (0) sein
	QWORD	qwLbaPosition;				// Position des eigenen LBA (dieses Headers)
	QWORD	qwBackupPosition;			// Position des Backup-LBA (des anderen Headers)
	QWORD	qwFirstLba;					// Erster benutzbarer LBA fr Partitionseintrge (normalerweise 34)
	QWORD	qwLastLba;					// Letzter benutzbarer LBA fr Partitionseintrge (normalerweise Datentrgergre - 34)
	GUID	sDiskGuid;					// Datentrger-GUID (als Referenz siehe auch UUID bei UNIXe)
	QWORD	qwPartitionTab;				// Start-LBA der Partitionstabelle (normalerweise 2)
	DWORD	dwUsedEntries;				// Anzahl der Partitionseintrge (Partitionen)
	DWORD	dwEntrySize;				// Gre eines Partitionseintrags (normalerweise 128)
	DWORD	dwPartitionCrc;				// Partitionstabellen-CRC32-Prfsumme
	BYTE	aUnused[420];				// Muss 0 sein
	}GuidPartitionTable;

typedef struct
	{
	GUID	sPartitionsTyp;				// Partitionstyp-GUID
	GUID	sPartitionsGuid;			// Einzigartige Partitions-GUID
	QWORD	qwStartLba;					// Beginn der Partition (erster LBA - Little-Endian)
	QWORD	qwEndLba;					// Ende der Partition (letzter LBA - inklusive)
	QWORD	qwAttributes;				// Attribute (siehe folgende Tabelle)
	WCHAR	aPartitionName[32];			// Partitionsname (UTF-16LE Code-Einheit)
	BYTE	aUnused[8];					// Unbenutzt
	}GuidPartitionEntry;

typedef struct
	{
	DWORD	dwSignature1;				// 0x41615252l
	BYTE	bReserved[480];
	DWORD	dwSignature2;				// 0x61417272l
	DWORD	dwFreeClusterCount;			// 0xFFFFFFFF for unkown, not guaranteed to be correct
	DWORD	dwNextFreeCluster;			// 0xFFFFFFFF for unkown, start free cluster search here, not guaranteed to be correct
	BYTE	bReserved2[12];
	DWORD	dwSignature;				// 0xAA550000l
	}FAT32InfoSector;

typedef struct
	{
	BYTE	bNearJump[3];				// 0xEB=Jump8	0xE9=Jump16
	char    cOEMName[8];
	WORD    wBytesPerSector;
	BYTE    bSectorsPerCluster;
	WORD    wReservedSectors;			// relative to partition start
	BYTE    bNumberOfFATs;
	WORD    wDirEntries;				// zero on FAT-32
	WORD    wSectorsOnDisk;
	BYTE    bMediaDescriptor;
	WORD    wSectorsPerFAT;				// zero on FAT-32
	WORD    wSectorsPerTrack;
	WORD    wNumberOfHeads;
	DWORD	dwNumberOfHiddenSectors;	// OS specific, unreliable
	DWORD	dwTotalSectors;
	BYTE    bPhysicalDiskNumber;		// 0x80, 0x81, etc.
	BYTE    bCurrentHead;				// not used
	BYTE    bSignature;					// needed by NT
	DWORD   dwSerialNumber;
	char    bLabel[11];					// not used
	char    cSystemID[8];
	BYTE	bBootCode[448];
	WORD	wSignature;					// 0xAA55
	}BootRecord16;


typedef struct
	{
	BYTE	bNearJump[3];				// 0xEB=Jump8	0xE9=Jump16
	char    cOEMName[8];
	WORD    wBytesPerSector;
	BYTE    bSectorsPerCluster;
	WORD    wReservedSectors;			// relative to partition start
	BYTE    bNumberOfFATs;
	WORD    wDirEntries;				// zero on FAT-32
	WORD    wSectorsOnDisk;
	BYTE    bMediaDescriptor;
	WORD    wSectorsPerFAT;				// zero on FAT-32
	WORD    wSectorsPerTrack;
	WORD    wNumberOfHeads;
	DWORD	dwNumberOfHiddenSectors;	// OS specific, unreliable
	DWORD	dwTotalSectors;
	DWORD	dwSectorsPerFAT;
	WORD    wFlags;						// if (Flags & 0x0080) { SingleFAT; ActiveFat = (Flags & 0x000F); }
	WORD    wVersion;
	DWORD	dwRootDirCluster;
	WORD    wFSInfoSector;				// relative to start of partition, usually 1, 0xFFFF for none.
	WORD    wBackupBootSector;			// usually 6
	BYTE    bReserved[12];				// should be all zero
	BYTE    bPhysicalDiskNumber;		// 0x80, 0x81, etc.
	BYTE    bCurrentHead;				// not used
	BYTE    bSignature;					// needed by NT
	DWORD   dwSerialNumber;
	char    bLabel[11];					// not used
	char    cSystemID[8];
	BYTE	bBootCode[420];
	WORD	wSignature;					// 0xAA55
	}BootRecord32;

typedef struct
	{
	BYTE	bNearJump[3];				// 0xEB=Jump8	0xE9=Jump16
	char    cOEMName[8];
	WORD    wBytesPerSector;
	BYTE    bSectorsPerCluster;
	WORD    wReservedSectors;			// relative to partition start
	BYTE    bReserved1[3];				// zero
	WORD    wSectorsOnDisk;
	BYTE    bMediaDescriptor;			// is 0xF8
	WORD    wReserved2;					// zero
	WORD    wSectorsPerTrack;
	WORD    wNumberOfHeads;
	DWORD	dwNumberOfHiddenSectors;	// OS specific, unreliable
	DWORD	dwReserved3;				// zero
	BYTE    bPhysicalDiskNumber;		// 0x80, 0x81, etc.
	BYTE    bCurrentHead;				// not used
	BYTE    bSignature;					// needed by NT
	BYTE	bReserved4;					// zero
	QWORD	qwTotalSectors;
	QWORD	qwMtfCluster;
	QWORD	qwMtfMirrorCluster;
	DWORD	dwClusterPerRecord;
	DWORD	dwClusterPerIndexBlock;
	QWORD	qwSerialNumber;
	DWORD	dwCheckSum;
	BYTE	bBootCode[426];
	WORD	wSignature;					// 0xAA55
	}BootRecordNtfs;

typedef struct
	{
	BYTE	bNearJump[3];				// 0xEB=Jump8	0xE9=Jump16
	char    cOEMName[8];				// always "EXFAT"
	WORD    wBytesPerSector;
	BYTE    bSectorsPerCluster;
	WORD    wReservedSectors;			// relative to partition start
	BYTE    bNumberOfFATs;
	WORD    wDirEntries;				// zero on FAT-32
	WORD    wSectorsOnDisk;
	BYTE    bMediaDescriptor;
	WORD    wSectorsPerFAT;				// zero on FAT-32
	WORD    wSectorsPerTrack;
	WORD    wNumberOfHeads;
	DWORD	dwNumberOfHiddenSectors;	// OS specific, unreliable
	DWORD	dwTotalSectors;
	DWORD	dwSectorsPerFAT;
	WORD    wFlags;						// if (Flags & 0x0080) { SingleFAT; ActiveFat = (Flags & 0x000F); }
	WORD    wVersion;
	DWORD	dwRootDirDummy;
	WORD    wFSInfoSector;				// relative to start of partition, usually 1, 0xFFFF for none.
	WORD    wBackupBootSector;			// usually 6
	BYTE    bReserved[12];				// should be all zero
	QWORD	qwSectorStart;				// 0x40 partition first sector
	QWORD	qwSectorCount;				// 0x48 partition sectors count
	DWORD	dwFatSectorStart;			// 0x50 FAT first sector
	DWORD	dwFatSectorCount;			// 0x54 FAT sectors count
	DWORD	dwClusterSectorStart;		// 0x58 first cluster sector
	DWORD	dwClusterCount;				// 0x5C total clusters count
	DWORD	dwRootDirCluster;			// 0x60 first cluster of the root dir
	DWORD	dwVolumeSerial;				// 0x64 volume serial number
	BYTE	bVersionMinor;				// minor version
	BYTE	bVersionMajor;				// major version
	WORD	wVolumeState;				// 0x6A volume state flags
	BYTE	bSectorBits;				// 0x6C sector size as (1 << n)
	BYTE	bSpcBits;					// 0x6D sectors per cluster as (1 << n)
	BYTE	bFatCount;					// 0x6E always 1
	BYTE	bDriveNumber;				// 0x6F always 0x80
	BYTE	bAllocatedPercent;			// 0x70 percentage of allocated space
	BYTE	bBootCode[397];				// 0x71 always 0
	WORD	wSignature;					// 0xAA55
	}BootRecordExFat;

typedef struct
	{
	BYTE	bLength;					// Lnge der Sructur
	BYTE	bExtendedSectors;			// Anzahl der erweiterten Sektoren am Beginn einer Datei
	DWORD	dwLsbSector;				// Startsektor der Datei
	DWORD	dwMsbSector;				// Startsektor der Datei (Motorola Format)
	DWORD	dwLsbFileLength;			// Lnge der Datei
	DWORD	dwMsbFileLength;			// Lnge der Datei
	BYTE	bYear;						// Jahr (+1900)
	BYTE	bMonth;						// Monat (1=Jan)
	BYTE	bDay;						// 1-31
	BYTE	bHour;						// Stunde
	BYTE	bMinute;					// Minute
	BYTE	bSecound;					// Sekunde
	BYTE	bTimezone;					// Zeitzone in 15min Schritten
	BYTE	bFlags;						//
	BYTE	bInterleaveSize;			// Bit: 0=Versteckt 1=Verzeichnis 7=nicht Ende
	BYTE	bInterleaveGAP;				//
	WORD	wLsbVolumeSequenz;			//
	WORD	wMsbVolumeSequenz;			//
	BYTE	bNameLength;				// Lnge des Dateinamens inklusive 0 am Ende
	char	cName[1];					// Dateiname (Rootverzeichnis="")
	}DirectoryRecord;

typedef struct
	{
	BYTE	bVDType;					// 1=primary volume descriptor 2=Secondary 0xFF=Terminator
	char	cVSStdId[5];				// Must be "CD001".
	BYTE	bVSStdVersion;           	// Must be 1.
	BYTE	bVolumeFlags;            	// 0 in primary volume descriptor.
	char	cSystemIdentifier[32];   	// What system this CD-ROM is meant for.
	char	cVolumeIdentifier[32];   	// The volume name.
	char	cReserved2[8];           	// Must be 0's.
	DWORD	dwLsbVolumeSpaceSize;     	// Volume size, least-significant-byte order.
	DWORD	dwMsbVolumeSpaceSize;     	// Volume size, most-significant-byte order.
	char	cEscapeSequences[32];    	// 0's in primary volume descriptor
	WORD	wLsbVolumeSetSize;       	// Number of volumes in volume set (must be 1).
	WORD	wMsbVolumeSetSize;
	WORD	wLsbVolumeSetSequenceNumber;// Which volume in volume set (not used).
	WORD	wMsbVolumeSetSequenceNumber;
	WORD	wLsbLogicalBlockSize;    	// We'll assume 2048 for block size.
	WORD	wMsbLogicalBlockSize;
	DWORD	dwLsbPathTableSize;       	// How many bytes in path table.
	DWORD	dwMsbPathTableSize;
	DWORD	dwLsbPathTable1;          	// Mandatory occurrence.
	DWORD	dwLsbPathTable2;          	// Optional occurrence.
	DWORD	dwMsbPathTable1;          	// Mandatory occurrence.
	DWORD	dwMsbPathTable2;          	// Optional occurrence.
	DirectoryRecord	sRootDirectory; 	// Duplicate root directory entry.
	char	cVolumeSetIdentifier[128];	// Various copyright and  control fields follow.
	char	cPublisherIdentifier[128];
	char	cDataPreparerIdentifier[128];
	char	cApplicationIdentifier[128];
	char	cCopyrightFileIdentifier[37];
	char	cAbstractFileIdentifier[37];
	char	cBibliographicFileIdentifier[37];
	char	cVolumeCreation[17];
	char	cVolumeModification[17];
	char	cVolumeExpiration[17];
	char	cVolumeEffective[17];
	BYTE	bFileStructureStandardVersion;
	char	cReserved4;               	// Must be 0.
	char	cApplicationUse[512];
	char	cFutureStandardization[653];
	}VolumeDescriptor;

typedef struct
	{
	char	cId[4];						// "FILE"
	WORD	wStart;						// Fixup start
	WORD	wCount;						// Fixup count
	QWORD	qwLigicalSequenzNumber;
	WORD	wSequenzNumber;
	WORD	wHardLinksCount;			// offset to data
	WORD	wAttributOffset;			// form	zero
	WORD	wFlags;
	DWORD	dwSize;						// from offset 8
	DWORD	dwTotalSize;
	QWORD	qwBaseMftRecort;
	WORD	wNextAttribut;
	}NtfsMftEntry;

typedef struct
	{
	DWORD	dwType;						// 10h = Standard Info
										// 20h = AttributList
										// 30h = Filename
										// 40h = Volume Version
										// 50h = Security Deskripor
										// 60h = Volume Name
										// 70h = Volume Info
										// 80h = Data
										// 90h = Index Root
										// A0h = Index Alloc
										// B0h = Bitmap
										// C0h = Sympolic Link
	DWORD	dwLength;					// Length of the attribut
	BYTE	bResisdent;					// 0 or 1
	BYTE	bNameLength;				// length of the wName field
	WORD	wNameOffest;				// offset of name
	WORD	wFlags;
	WORD	wAttrNumber;
	WORD	wDataSize;					// size   of data
	WORD	wUnknown;
	WORD	wDataOffest;				// offset of data
	WORD	wIndex;
	}NtfsAttribut;

typedef struct							// Type 0x30
	{
	DWORD	dwINumberLow;
	WORD	wINumberHigh;
	WORD	wSequenzNumber;
	QWORD	qwCreationTime;
	QWORD	qwLastMotifyTime;
	QWORD	qwLastMftMotifyTime;
	QWORD	qwAccessTime;
	QWORD	qwAllocSize;
	QWORD	qwDataSize;
	DWORD	dwFlags;
	DWORD	dwReserved;
	BYTE	bNameLength;
	BYTE	bNameLengthPosix;
	WORD	wName[1];
	}NtfsAttributFile;

typedef struct 							// Type 0x10
	{
	QWORD	qwCreationTime;
	QWORD	qwLastMotifyTime;
	QWORD	qwLastMftMotifyTime;
	QWORD	qwAccessTime;
	QWORD	qwReserved;
	}NtfsAttributInfo;

typedef struct							// Type 0x80
	{
	DWORD	dwType;						// Type 0x80
	DWORD	dwLength;					// Length of the attribut
	QWORD	qwUnknown;
	QWORD	qwStartCluster;
	QWORD	qwEndCluster;
	WORD	wOffset;
	WORD	wReserved;
	DWORD	dwReserved;
	QWORD	qwLength;
	}NtfsAttributData;

typedef struct
	{
	WORD	wTagIdentifier;				// Art des Tags
	WORD	wDescVersion;				// Version des Tags
	BYTE	bChecksum;					// Checksumme des Tags
	BYTE	bReserved;					// Unbenutzt
	WORD	wSerialNumber;				// eriennummer
	WORD	wDescCrc;					// Checksumme fr Descriptor
	WORD	wDescLen;					// Descriptor Lnge
	DWORD	dwLocation;					// Logischer Sektor vom Inhalt
	}EcmaTag;

typedef struct
	{
	DWORD	dwDirectEntries;			// Prior Recorded Number of Direct Entries
	WORD	wStrategyType;				// Strategy Type
	WORD	wStrategyParam;				// Strategy Parameter
	WORD	wMaxNumberOfEntries;		// Maximum Number of Entries
	BYTE	bReserved;					// Reserved
	BYTE	bFileType;					// File Type
	BYTE	aParentIcbLoc[6];			// Parent ICB Location
	WORD	wFlags;						// Flags
	}EcmaIcb;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=1)
	DWORD	dwSequenceNumber;			// Volume Descriptor Sequence Number
	DWORD	dwDescriptorNumber;			// Primary Volume Descriptor Number
	BYTE	aIdentifierString[32];		// Identifier String
	WORD	wSequenceNumber;			// Volume Sequence Number
	WORD	wMaxVolumeSequence;			// Maximum Volume Sequence Number
	WORD	wInterchangeLevel;			// Interchange Level
	WORD	wMaxInterchange;			// Maximum Interchange Level
	DWORD	dwCharSetList;				// Character Set List
	DWORD	dwMaxCharSetList;			// Maximum Character Set List
	BYTE	aVolumeSetIdentifier[128];	// Volume Set Identifier
	BYTE	aDescriptorCharSet[64];		// Descriptor Character Set
	BYTE	aExplanatoryCharSet[64];	// Explanatory Character Set
	DWORD	dwVolumeAbstractLen;		// Volume Abstract
	DWORD	dwVolumeAbstractLoc;		// Volume Abstract
	DWORD	dwVolumeCopyrightLen;		// Volume Copyright Notice
	DWORD	dwVolumeCopyrightLoc;		// Volume Copyright Notice
	BYTE	aApplicationIdentifier[32];	// Application Identifier
	BYTE	aRecordingDateTime[12];		// Recording Date and Time
	BYTE	aImplementIdentifier[32];	// Implementation Identifier
	BYTE	aImplementationUse[64];		// Implementation Use bytes
	DWORD	dwSequenceLocation;			// Predecessor Volume Descriptor
	WORD	wFlags;						// Flags
	BYTE	aReserved[22];				// Reserved
	}EcmaPrimVolumeDesc;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=2)
	DWORD	dwMainVolumeDescLen;		// Main Volume Descriptor
	DWORD	dwMainVolumeDescLoc;		// Main Volume Descriptor
	DWORD	dwReserveVolumeDescLen;		// Reserve Volume Descriptor
	DWORD	dwReserveVolumeDescLoc;		// Reserve Volume Descriptor
	BYTE	aReserved[480];				// Reserved
	}EcmaAnchorVolumeDesc;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=3)
	DWORD	dwSequenceNumber;			// Volume Descriptor Sequence Number
	DWORD	dwNexSequenceExtentLen;		// Next Volume Descriptor Sequence Extent
	DWORD	dwNexSequenceExtentLoc;		// Next Volume Descriptor Sequence Extent
	BYTE	aReserved[484];				// Reserved
	}EcmaVolumeDesc;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=4)
	DWORD	dwSequenceNumber;			// Volume Descriptor Sequence Number
	BYTE	aImplementationId[32];		// Implementation Identifier
	BYTE	aUserdata[460];				// Userdata
	}EcmaImplementationUse;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=5)
	DWORD	dwSequenceNumber;			// Volume Descriptor Sequence Number
	WORD	wPartitionFlags;			// Partition Flags
	WORD	wPartitionNumber;			// Partition Number
	BYTE	aPartitionContents[32];		// Partition Contents
	BYTE	aPartitionContentsUse[128];	// Partition Contents Use
	DWORD	dwAccessType;				// Access Type
	DWORD	dwPartitionLoc;				// Partition Starting Location
	DWORD	dwPartitionLen;				// Partition Length
	BYTE	aImplementationId[32];		// Implementation Identifier
	BYTE	aImplementationUse[128];	// Implementation Use
	BYTE	aReserved[356];				// Reserved
	}EcmaPartitionDesc;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=6)
	DWORD	dwSequenceNumber;			// Volume Descriptor Sequence Number
	BYTE	aCharacterSet[64];			// Descriptor Character Set
	BYTE	aVolumeIdentifier[128];		// Logical Volume Identifier
	DWORD	dwLogicalBlockSize;			// Logical Block Size
	BYTE	aDomainIdentifier[32];		// Domain Identifier
	BYTE	aVolumeContentsUse[16];		// Logical Volume Contents Use
	DWORD	dwMapTableLength;			// Map Table Length
	DWORD	dwNumberPartitionMaps;		// Number of Partition Maps
	BYTE	bImplementationId[32];		// Implementation Identifier
	BYTE	bImplementationUse[128];	// Implementation Use
	DWORD	dwIntegritySequenceLoc;		// Integrity Sequence Extent
	DWORD	dwIntegritySequenceLen;		// Integrity Sequence Extent
	BYTE	bReserved[440];				// Reserved
	}EcmaLogicalVolumeDesc;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=7)
	DWORD	dwSequenceNumber;			// Volume Descriptor Sequence Number
	DWORD	dwNumberOfAllocation;		// Number of Allocation Descriptors
	DWORD	aAllocationDesc[2];			// Allocation Descriptor
	}EcmaUnallocatedSpaceDesc;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=8)
	BYTE	bReserved[496];				// Reserved
	}EcmaTerminatingDesc;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=9)
	BYTE	aRecordingTime[16];			// Recording Date and Time
	DWORD	dwIntegrityType;			// Integrity Type
	DWORD	dwNextIntegrityLen;			// Next Integrity
	DWORD	dwNextIntegrityLoc;			// Next Integrity
	BYTE	aVolumeContentsUse[32];		// Logical Volume Contents Use
	DWORD	dwNumberOfPartitions;		// Number of Partitions  N_P
	DWORD	dwLengthOfImplementation;	// Length of Implementation
	DWORD	aData[1];					// FreeSpaceTable[N_P]+SizeTable[N_P*]+...Implementation Use
	}EcmaLogicalVolumeInt;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=256)
	BYTE	aRecordingTime[12];			// Recording Date and Time
	WORD	wInterchangeLevel;			// Interchange Level
	WORD	wMaxInterchangeLevel;		// Max.Interchange Level
	DWORD	dwCharacterSetList;			// Character Set List
	DWORD	dwMaxCharacterSetList;		// Max. Character Set List
	DWORD	dwFileSetNumber;			// File Set Number
	DWORD	dwFileSetDescNumber;		// File Set Descriptor Number
	BYTE	aVolumeIdentCharSet[64];	// Logical Volume Identifier Character Set
	BYTE	aVolumeIdentifier[128];		// Logical Volume Identifier
	BYTE	aFileSetCharSet[64];		// File Set Character Set
	BYTE	aFileSetIdentifier[32];		// File Set Identifier
	BYTE	aCopyrightIdentifier[32];	// Copyright File Identifier
	BYTE	aAbstractIdentifier[32];	// Root Directory ICB
	BYTE	aRootDirectoryIcb[16];		// Root Directory ICB
	BYTE	aDomainIdentifier[32];		// Domain Identifier
	BYTE	aNextExtentIcb[16];			// Next Extent
	BYTE	aSystemStreamIcb[16];		// System Stream Directory ICB
	BYTE	bReserved[32];				// Reserved
	}EcmaFileSetDesc;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=257)
	WORD	wFileVersionNumber;			// File Version Number
	BYTE	bFileCharacteristics;		// File Characteristics
	BYTE	bLengthOfFileIdentifier;	// Length of File Identifier L_FI
	BYTE	aFileIcb[16];				// File ICB
	WORD	wImplementationUseLen;		// Length of Implementation Use L_IU
	BYTE	aData[1];					// ImplementationUse[L_IU]+FileIdentifier[L_FI]
	}EcmaFileIdentifierDesc;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=258)
	DWORD	dwAllocExtentLoc;			// Previous Allocation Extent Location
	DWORD	dwLengthOfAllocation;		// Length of Allocation Descriptors
	}EcmaAllocExtentDesc;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=259)
	BYTE	aIcbTag[16];				// ICB Tag
	BYTE	aIndirectIcb[16];			// Indirect ICB
	}EcmaIndirectEntry;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=260)
	BYTE	aIcbTag[16];				// ICB Tag
	}EcmaTerminalEntry;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=261)
	BYTE	aIcbTag[20];				// ICB Tag
	DWORD	dwUid;						// Uid
	DWORD	dwGid;						// Gid
	DWORD	dwPermissions;				// Permissions
	WORD	wFileLinkCount;				// File Link Count
	BYTE	bRecordFormat;				// Record Format
	BYTE	bRecordAttributes;			// Record Display Attributes
	DWORD	dwRecordLength;				// Record Length
	QWORD	qwInformationLength;		// Information Length
	QWORD	qwLogicalBlocksRecorded;	// Logical Blocks Recorded
	BYTE	aAccessTime[12];			// Access Date and Time
	BYTE	aModificationTime[12];		// Modification Date and Time
	BYTE	aAttributeTime[12];			// Attribute Date and Time
	DWORD	dwCheckpoint;				// Checkpoint
	BYTE	aExtendedAttributeIcb[16];	// Extended Attribute ICB
	BYTE	aImplementationId[32];		// Implementation Identifier
	QWORD	qwUniqueId;					// Unique Id
	DWORD	dwLengthOfExAttributes;		// Length of Extended Attributes L_EA
	DWORD	dwLengthOfAllocation;		// Length of Allocation Descriptors	L_AD
	BYTE	aData[1];					// ExtendedAttributes[L_EA]+AllocationDescriptors[L_AD]
	}EcmaFileEntry;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=262)
	DWORD	dwImplAttrLocation;			// Implementation Attributes Location
	DWORD	dwApplAttrLocation;			// Application Attributes Location
	}EcmaExtendedAttrHeader;

typedef struct
	{
	BYTE	bType;						// EXFAT_ENTRY_BITMAP (0x01)
	BYTE	aUnused[19];				// Unused data
	DWORD	dwStartCluster;				// Start cluster of the file or directory
	QWORD	qwSize;						// Size of file
	}ExFatBitmap;

typedef struct 							// upper case translation table
	{
	BYTE	bType;						// EXFAT_ENTRY_UPCASE (0x02)
	BYTE	aUnused1[3];				// Unused data
	DWORD	dwCheckSum;					// Checksumme
	BYTE	aUnused2[12];				// Unused data
	DWORD	dwStartCluster;				// Start cluster of the item
	QWORD	qwSize;						// Size of item
	}ExFatUpcase;

typedef struct
	{
	BYTE	bType;						// EXFAT_ENTRY_LABEL (0x03)
	BYTE	bLength; 					// number of characters
	WORD	aName[15];					// in UTF-16LE
	}ExFatLabel;

typedef struct
	{
	BYTE	bType;						// EXFAT_ENTRY_FILE  (0x05)
	BYTE	bContinuations;				//
	WORD	wCheckSum;					// Checksum
	WORD	wAttributes; 				// combination of EXFAT_ATTRIB_xxx
	WORD	wUnused1;					// Unused data
	WORD	wCreateTime; 				// creation date and time
	WORD	wCreateDate;				//		*
	WORD	wWriteTime;					// latest modification date and time
	WORD	wWriteDate;					//		*
	WORD	wAcessTime;					// latest access date and time
	WORD	wAcessDate;					//		*
	BYTE	wCreateCenti;				// creation time in cs (centiseconds)
	BYTE	wWriteCenti;				// latest modification time in cs
	BYTE	aUnused2[10];				// Unused data
	}ExFatFile;

typedef struct
	{
	BYTE	bType;						// EXFAT_ENTRY_FILE_INFO (0x00)
	BYTE	bFlags;						// combination of EXFAT_FLAG_xxx
	BYTE	bUnused1;					// Unused data
	BYTE	bNameLength;				// legth of the file name
	WORD	wNameHash;					// Hsh of the filename
	WORD	wUnused2;					// Unused data
	QWORD	qwValidSize; 				// in bytes, less or equal to size
	BYTE	bUnused3[4];				// Unused data
	DWORD	dwStartCluster;				// Start cluster of the item
	QWORD	qwSize;						// Size of item
	}ExFatInfo;

typedef struct
	{
	BYTE	bType;						// EXFAT_ENTRY_FILE_NAME (0x01)
	BYTE	bUnused;					// Unused data
	WORD	aName[15];					// in UTF-16LE
	}ExFatName;

typedef struct
	{
	DWORD	dwInodesCount;				// I-nodes count
	DWORD	dwBlocksCountLo;			// Blocks count
	DWORD	dwReservedBlocksCountLo;	// Reserved blocks count
	DWORD	dwFreeBlocksCountLo;   		// Free blocks count
	DWORD	dwFreeInodesCount;			// Free inodes count
	DWORD	dwFirstDataBlock;	 		// First Data Block
	DWORD	dwLogBlockSize;				// Block size (0x400<<n)
	DWORD	dwLogClusterSize;	 		// Obsoleted fragment size
	DWORD	dwBlocksPerGroup;	 		// Number of blocks per group
	DWORD	dwFragsPerGroup;	  		// Obsoleted fragments per group
	DWORD	dwInodesPerGroup;	 		// Number of inodes per group
	DWORD	dwMountTime;		   		// Mount time
	DWORD	dwWriteTime;		   		// Write time
	WORD	wMountCount;		   		// Mount count
	WORD	wMaxMountCount;				// Maximal mount count
	WORD	wMagic;			   			// Magic signature 0xEF53
	WORD	wState;			   			// File system state
	WORD	wErrors;		   			// Behavior when detecting errors
	WORD	wMinorRevLevel;				// Minor revision level
	DWORD	dwLastCheckTime;			// Time of last check
	DWORD	dwCheckInterval;			// Maximum time between checks
	DWORD	dwCreatorOs;				// Creator OS
	DWORD	dwRevLevel;					// Revision level
	WORD	wDefResUid;					// Default uid for reserved blocks
	WORD	wDefResGid;					// Default gid for reserved blocks
	DWORD	dwFirstInode;				// First non-reserved inode
	WORD	wInodeSize;					// Size of inode structure
	WORD	wBlockGroupIndex;   		// Block group index of this superblock
	DWORD	dwFeaturesCompatible; 		// Compatible feature set
	DWORD	dwFeaturesIncompatible;  	// Incompatible feature set
	DWORD	dwFeaturesReadOnly;     	// Readonly-compatible feature set
	BYTE	aUuid[16];					// 128-bit uuid for volume
	CHAR	cVolumeName[16];		 	// Volume name
	CHAR	cLastMounted[64];		 	// Directory where last mounted
	DWORD	dwAlgorithmUsageBitmap; 	// For compression
	BYTE 	bPreAllocBlocks;			// Number of blocks to try to preallocate
	BYTE 	bPreAllocDirBlocks; 		// Number to preallocate for dirs
	WORD 	bReservedGdtBlocks;			// Per group desc for online growth
	BYTE	aJournalUuid[16];   		// UUID of journal superblock
	DWORD 	dwJournalInodeNumber;		// Inode number of journal file
	DWORD 	dwJournalDevice;			// Device number of journal file
	DWORD 	dwLastOrphan;				// Head of list of inodes to delete
	DWORD 	aHashSeed[4];				// HTREE hash seed
	BYTE 	bDefaultHashVersion;		// Default hash version to use
	BYTE	bJournalBackupType;			// If zero or 1, then contains a duplicate copy of the inode's
	WORD 	wDescSize;					// Size of group descriptor
	DWORD 	dwDefaultMountOpts; 		// Default mount options
	DWORD 	dwFirstMetaBg;    			// First metablock block group
	DWORD 	dwMkfsTime;					// When the filesystem was created
	DWORD 	aJournalBlocks[17]; 		// Backup of the journal inode
	DWORD 	dwBlocksCountHi;			// Blocks count
	DWORD 	dwReservedBlocksCountHi;	// Reserved blocks count
	DWORD 	dwFreeBlocksCountHi;		// Free blocks count
	WORD 	wMinExtraInodeSize;			// All inodes have at least # bytes
	WORD 	wWantExtraIsize;			// New inodes should reserve # bytes
	DWORD 	dwFlags;					// Miscellaneous flags
	WORD 	wRaidStride;				// RAID stride
	WORD 	wMmpInterval;				// # seconds to wait in MMP checking
	QWORD 	qwMmpBlock;	  				// Block for multi-mount protection
	DWORD 	dwRaidStripeWidth;			// Blocks on all data disks (N * stride)
	BYTE 	bLogGroupsPerFlex;			// FLEX_BG group size
	BYTE 	bChecksumType;				// Metadata checksum algorithm type
	WORD 	wReservedPad;				// Unused
	QWORD 	qwKbytesWritten;			// Number of lifetime kilobytes written
	DWORD 	dwSnapshotInode;			// I-node number of active snapshot
	DWORD 	dwSnapshotId;				// Sequential ID of active snapshot
	QWORD	qwSnapshotBlocksCount;		// Reserved blocks for active snapshot's future use
	DWORD	dwSnapshotList;				// I-node number of the head of the on-disk snapshot list
	DWORD 	dwErrorCount;				// Number of file system errors
	DWORD 	dwFirstErrorTime;			// First time an error happened
	DWORD 	dwFirstErrorInode;			// I-node involved in first error
	QWORD 	qwFirstErrorBlock;			// Block involved of first error
	CHAR 	cFirstErrorFunc[32];		// Function where the error happened
	DWORD 	dwFirstErrorLine;			// Line number where error happened
	DWORD 	dwLastErrorTime;			// Most recent time of an error
	DWORD 	dwLastErrorInode;			// I-node involved in last error
	DWORD 	dwLastErrorLine;			// Line number where error happened
	QWORD 	qwLastErrorBlock;			// Block involved of last error
	CHAR 	cLastErrorFunc[32];			// Function where the error happened
	CHAR 	cMountOpts[64];				// ASCII string of mount options.
	DWORD 	dwUserQuotaInode;			// inode for tracking user quota
	DWORD 	dwGroupQuotaInode;			// inode for tracking group quota
	DWORD 	dwOverheadClusters;			// overhead blocks/clusters in fs
	DWORD 	dwBackupBgs[2];				// groups with sparse_super2 SBs
	BYTE  	bEncryptAlgorithms[4];		// Encryption algorithms in use
	BYTE  	bEncryptPasswortSalt[16];	// Salt used for string2key algorithm
	DWORD 	dwLpfInode;					// Location of the lost+found inode
	DWORD 	dwPadding[100];				// Padding to the end of the block
	DWORD 	dwChecksum;					// crc32c(superblock)
	}ExtSuperBlock;

#pragma pack(pop)


//*****************************************************************************
//*
//*		Klasse	CDiskDoc
//*
//*****************************************************************************
class CDiskDoc : public CDocument
{

protected: // Nur aus Serialisierung erzeugen

	CDiskDoc();
	DECLARE_DYNCREATE(CDiskDoc)
	DECLARE_LOAD_LAST()

public:

	LPBYTE  GetData();								// Zeiger auf Sektor Daten holen
	DWORD	GetDrive();								// Laufwerks Nummer (0=A 1=B ...)
	DWORD	GetSectorType();						// Ergibt den Standard Type des Sektors
	DWORD	GetSectorSize();						// Gre des Sektors
	DWORD	GetSectorState();						// Zustand 0=geladen 1=leer 2=Fehler
	QWORD	GetSectorNumber();						// Nummer des geladenen Sektors
	QWORD	GetPartitionBase();						// Start des Partition abfragen
	QWORD	GetSectorCount();						// Anzahl der 512 Byte Sektoren
	QWORD	GetFATStart();							// Startsektor der ersten FAT
	DWORD	GetFATSize();							// Gre einer FAT-Tabelle
	QWORD	GetClusterMin();						// Erste Cluster-Nummer abfragen
	QWORD	GetClusterStart();						// Sektornummer des ersten Clusters
	DWORD	GetClusterSize();						// Gre eines Clusters in Sektoren
	DWORD	GetMftData(void *pData,void **pBase);	// Einen MFT Block laden
	LPCSTR	GetEmulationName();						// Ergibt den Namen der Image Datei
	BOOL	GetReadSectors(QWORD qwStart,DWORD dwCount,void *pBuffer);

	static int DecodeClusterData(long lLen,BYTE *pData,DWORD *pCluster,DWORD *pCount,int iMax);

	//{{AFX_VIRTUAL(CDiskDoc)
public:
	virtual BOOL   OnNewDocument();
	virtual BOOL   OnOpenDocument(LPCTSTR    pPathName);
	virtual void   PreCloseFrame (CFrameWnd *pFrame );
	virtual void   Serialize(CArchive &ar);
	//}}AFX_VIRTUAL

public:
	virtual ~CDiskDoc();

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

	DWORD		dwDrive;							// Laufwerks Nummer (0=A 1=B ...)
	DWORD		dwDriveOld;							// Alte Laufwerks Nummer
	BYTE		bSectorData[0x10000];				// Sektor Daten
	BYTE		bSectorOld [0x10000];				// Sektor Daten ohne nderungen
	DWORD		dwSectorSize;						// Gre des Sektors
	DWORD		dwSectorState;						// Zustand 0=geladen 1=leer 2=Fehler
	QWORD		qwSectorNumber;						// Nummer des geladenen Sektors
	QWORD		qwSectorCount;						// Anzahl der Sektoren am Laufwerk
	DWORD		dwDriveSectorSize;					// Gre der Sektoren am Laufwerk
	DWORD		dwDriveMul;							// Gre der Sektoren in 512 BYTE Blcken
	BOOL		bIsDriveOk;							// Ist ein Laufwerk vorhanden
	BOOL		bIsWriteable;						// Kann der Sektor verndert werden.
	BOOL		bIsDisk;							// Ist es eine Festplatte

	DWORD		dwSerialNumber;						// Volume Seriennummer
	DWORD		dwFilenameLength;					// Lnge der Dateinamen
	DWORD		dwFileSystemFlags;					// Dateisystem Flags
	char		cVolume[63];						// Volume Name
	char		cSystem[32];						// Dateisystem

	QWORD		qwFATStart;							// Startsektor der ersten FAT
	DWORD		dwFATSize;							// Gre einer FAT-Tabelle

	DWORD		dwClusterParts; 					// Anzahl der Partitionen
	QWORD		qwClusters     [PARTS_MAX];			// Anzahl der Cluster
	DWORD		dwClusterSize  [PARTS_MAX];			// Gre eines Clusters in Sektoren
	QWORD		qwClusterStart [PARTS_MAX];			// Erster Sektor der Cluster
	QWORD		qwClusterFrom  [PARTS_MAX];			// Erster Sektor der Partition
	QWORD		qwClusterMin   [PARTS_MAX];			// Niedrigste Clusternummer

	DWORD		dwPartitions;						// Anzahl der Partitionen
	QWORD		qwPartitionBase[PARTS_MAX];			// Anfang der Partitionen

	DWORD		dwSecUnits;							// Benutzte Sektoreinheiten
	QWORD		qwSecPos[UNITS_MAX];				// Sektor Positionen
	QWORD		qwSecEnd[UNITS_MAX];				// Sektor Lngen
	DWORD		dwSecTyp[UNITS_MAX];				// Sektor Typen
	LPCSTR		aSecName[UNITS_MAX];				// Sektor Namen

   	int			iFindFlags;							// Suchendaten
	int			iOpenMode;							// 0=C: ffnen 1=D: ffnen 2=nichts ffnen 3=letzes ffnen
	char	   *pFindSuchen;
	char	   *pFindGruppe;
	char	   *pFindErgebnis;
	QWORD		qwFindStart;						// Startsektor fr die Suche
	QWORD		qwFindCount;						// Sektorenanzahl fr die Suche
	QWORD		qwFindSecPos;						// Jetziger Sektor in dem gesucht wird
	QWORD		qwFindPos;							// Jetzige Position im Sektor
	BYTE	   *pFindData;							// Daten mit geladen Sektoren
	QWORD		qwFindDataSec;						// Startsektor der Daten
	QWORD		qwFindDataSize;						// Anzahl der geladen Sektoren

	QWORD		qwFindResultSec;					// Das Ergebnis eine Sektorsuche
	QWORD		qwFindResultPos;					//			*
	DWORD		dwFindResultAnz;					//			*

	HANDLE		hEmulation;							// Handle fr die Emulations Datei
	CString		sEmulation;							// Name der Emulations Datei


	BOOL	IsModified();
	BOOL	UpdateSector(QWORD qwPos=0,DWORD qwLen=0);
	void	InitDocument();
	void	SortEntries();
	int		MessageBox( LPCTSTR lpszText, LPCTSTR lpszCaption = NULL, UINT nType = MB_OK );

	static  void _cdecl FindBreak(CProgressDlg  *pDlg);
		    int		    FindNext (CProgressDlg  *pDlg,CHexFind *pFind);
	static  int  _cdecl FindNext (CProgressDlg  *pDlg);
			int			SaveSec  (CProgressDlg  *pDlg,FILE *pFile,QWORD	qwSector,DWORD dwSize);
	static  int  _cdecl SaveSec  (CProgressDlg  *pDlg);
			int			LoadSec  (CProgressDlg  *pDlg,FILE *pFile,QWORD	qwSector,DWORD dwSize);
	static  int  _cdecl LoadSec  (CProgressDlg  *pDlg);

	static  BYTE		EcmaTagCrc(void *pTagPtr);
	static  LPCSTR		EcmaName  (void *pTagPtr);

	BOOL	ScanDrive();
	BOOL	ScanMBR	 (QWORD qwStart=0);
	BOOL	ScanNTFS (QWORD qwStart=0);
	BOOL	ScanCDFS (QWORD qwStart=0);
	BOOL	ScanExFAT(QWORD qwStart=0);
	BOOL	ScanFAT  (QWORD qwStart=0);
	BOOL	ScanUDF  (QWORD qwStart=0);
	BOOL	ScanEXT  (QWORD qwStart=0);
	BOOL	ScanFile (int   iDrive);

	BOOL	ReadSector	  (int iDrive,QWORD qwStart,DWORD dwCount,void *pBuffer);
	BOOL	WriteSector	  (int iDrive,QWORD qwStart,DWORD dwCount,void *pBuffer);
	QWORD	GetSectorCount(int iDrive,DWORD *pSize=0);


	//{{AFX_MSG(CDiskDoc)
	afx_msg	void OnDriveSelect();
	afx_msg	void OnSectorGoto();
	afx_msg	void OnSectorNext();
	afx_msg	void OnSectorBefore();
	afx_msg	void OnGotoCluster();
	afx_msg	void OnGoto();
	afx_msg	void OnFind();
	afx_msg	void OnFindNext();
	afx_msg	void OnSaveSec();
	afx_msg	void OnLoadSec();
	afx_msg	void OnLoadImage();
	afx_msg	void OnStartOptions();
	afx_msg	void OnPartitionLoad();
	afx_msg	void OnWriteProtection();
	afx_msg	void OnGotoOk(CCmdUI *pCmdUI);
	afx_msg	void OnFindNextOk(CCmdUI *pCmdUI);
	afx_msg	void OnSectorGotoOk(CCmdUI *pCmdUI);
	afx_msg	void OnSectorNextOk(CCmdUI *pCmdUI);
	afx_msg	void OnSectorBeforeOk(CCmdUI *pCmdUI);
	afx_msg	void OnGotoClusterOk(CCmdUI *pCmdUI);
	afx_msg	void OnPartitionLoadOk(CCmdUI *pCmdUI);
	afx_msg	void OnWriteProtectionOk(CCmdUI *pCmdUI);
	//}}AFX_MSG



	DECLARE_MESSAGE_MAP()
};

//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
inline	LPBYTE  CDiskDoc::GetData			(){return bSectorData;}
inline	LPCSTR	CDiskDoc::GetEmulationName  (){return sEmulation;}
inline	DWORD	CDiskDoc::GetDrive			(){return dwDrive;}
inline	DWORD	CDiskDoc::GetSectorSize		(){return dwSectorSize;}
inline	DWORD	CDiskDoc::GetSectorState	(){return dwSectorState;}
inline	QWORD	CDiskDoc::GetSectorNumber	(){return qwSectorNumber;}
inline	QWORD	CDiskDoc::GetFATStart		(){return qwFATStart;}
inline	DWORD	CDiskDoc::GetFATSize		(){return dwFATSize;}
inline	QWORD	CDiskDoc::GetSectorCount    (){return (dwDriveMul)? qwSectorCount/(QWORD)dwDriveMul:0;}
inline	BOOL	CDiskDoc::GetReadSectors    (QWORD qwStart,DWORD dwCount,void *pBuffer){return ReadSector(dwDrive,qwStart,dwCount,pBuffer);}
///*****************************************************************************
//{{AFX_INSERT_LOCATION
//}}AFX_INSERT_LOCATION

#endif


